package com.company.untitled.screen.activity;

import com.company.untitled.entity.Activity;
import com.company.untitled.entity.Group;
import io.jmix.core.EntityStates;
import io.jmix.ui.Notifications;
import io.jmix.ui.component.HasValue;
import io.jmix.ui.component.TextField;
import io.jmix.ui.model.InstanceContainer;
import io.jmix.ui.screen.*;
import org.springframework.beans.factory.annotation.Autowired;

import java.math.BigDecimal;
import java.util.Comparator;
import java.util.List;
import java.util.Optional;

@UiController("Activity.edit")
@UiDescriptor("activity-edit.xml")
@EditedEntityContainer("activityDc")
public class ActivityEdit extends StandardEditor<Activity> {
    @Autowired
    private EntityStates entityStates;
    @Autowired
    private Notifications notifications;
    @Autowired
    private TextField<BigDecimal> progressField;
    @Autowired
    private InstanceContainer<Activity> activityDc;

    @Subscribe
    public void onBeforeCommitChanges(final BeforeCommitChangesEvent event) {
        if(entityStates.isNew(getEditedEntity())){
            setSerialNumberToActivity();
        }
    }


    public void setSerialNumberToActivity() {
        // Before Commit, If new entity, then assign serial number to activity
        Group ag = getEditedEntity().getGroup();
        List<Activity> aa = ag.getActivities();
        if (aa == null){
            getEditedEntity().setSerialNumber(1);
        } else {
            if (aa.isEmpty()) {
                getEditedEntity().setSerialNumber(1);
            } else {
                Optional<Activity> cc = aa.stream().max(Comparator.comparingInt(Activity::getSerialNumber));
                getEditedEntity().setSerialNumber(cc.get().getSerialNumber() + 1);
            }
        }
    }

    @Subscribe("progressField")
    public void onProgressFieldValueChange(final HasValue.ValueChangeEvent<BigDecimal> event) {
        // TODO if we update progress of Excavation to 5 and then we want to further want to update progress of PCC to 3 it still read progress of excavation as 0
        List<Activity> aa = getEditedEntity().getGroup().getActivities();
        //List<Activity> aa =activityDc.getItem().getGroup().getActivities();
        if(getEditedEntity().getPredecessors() != null) {
            int pred = Integer.parseInt(getEditedEntity().getPredecessors());
            Optional<Activity> preActivity = aa.stream().filter(act -> act.getSerialNumber().equals(pred)).findFirst();
            BigDecimal prevActivityProgress = preActivity.get().getProgress();
            notifications.create().withCaption(prevActivityProgress.toString()).show();
            if (progressField.getValue().doubleValue() > prevActivityProgress.doubleValue()) {
                notifications.create().withType(Notifications.NotificationType.ERROR).withCaption("Progress can not be more than progress in predecessor activity").show();
                progressField.setValue(event.getPrevValue());
            }
        }

    }
    
}