package com.company.untitled.screen.group;

import com.company.untitled.entity.Activity;
import com.company.untitled.entity.Group;
import io.jmix.ui.Notifications;
import io.jmix.ui.component.Button;
import io.jmix.ui.component.Table;
import io.jmix.ui.screen.*;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.Comparator;
import java.util.List;
import java.util.Optional;

@UiController("Group_.edit")
@UiDescriptor("group-edit.xml")
@EditedEntityContainer("groupDc")
public class GroupEdit extends StandardEditor<Group> {
    @Autowired
    private Table<Activity> activitiesTable;
    @Autowired
    private Notifications notifications;

    @Subscribe("btnMoveDown")
    public void onBtnMoveDownClick(final Button.ClickEvent event) {
        if (activitiesTable.getSingleSelected() == null) {
            notifications.create().withCaption("First Select an Activity").show();
        } else {
            int selectedItemSN = activitiesTable.getSingleSelected().getSerialNumber();
            Optional<Activity> cc = activitiesTable.getItems().getItems().stream().max(Comparator.comparingInt(Activity::getSerialNumber));
            int maxSN = cc.get().getSerialNumber() ;
            if (selectedItemSN == maxSN) {
                notifications.create().withCaption("Already Last Item, Cannot move down").show();
            } else {
                Optional<Activity> selActivity = activitiesTable.getItems().getItems().stream().filter(act -> act.getSerialNumber().equals(selectedItemSN)).findFirst();
                Optional<Activity> nextActivity = activitiesTable.getItems().getItems().stream().filter(act -> act.getSerialNumber().equals(selectedItemSN + 1)).findFirst();
                selActivity.get().setSerialNumber(selectedItemSN + 1);
                nextActivity.get().setSerialNumber(selectedItemSN);
                clearPredecessors();
                activitiesTable.sort("serialNumber", Table.SortDirection.ASCENDING);
            }
        }
    }

    @Subscribe("btnMoveUp")
    public void onBtnMoveUpClick(final Button.ClickEvent event) {
        if (activitiesTable.getSingleSelected() == null) {
            notifications.create().withCaption("First Select an Activity").show();
        } else {
            int selectedItemSN = activitiesTable.getSingleSelected().getSerialNumber();
            if (selectedItemSN == 1) {
                notifications.create().withCaption("Already First Item, Cannot move up").show();
            } else {
                Optional<Activity> selActivity = activitiesTable.getItems().getItems().stream().filter(act -> act.getSerialNumber().equals(selectedItemSN)).findFirst();
                Optional<Activity> prevActivity = activitiesTable.getItems().getItems().stream().filter(act -> act.getSerialNumber().equals(selectedItemSN - 1)).findFirst();
                selActivity.get().setSerialNumber(selectedItemSN - 1);
                prevActivity.get().setSerialNumber(selectedItemSN);
                clearPredecessors();
                activitiesTable.sort("serialNumber", Table.SortDirection.ASCENDING);
            }
        }
    }

    private void clearPredecessors(){
        List<Activity> listAct = getEditedEntity().getActivities();
        if (!listAct.isEmpty()){
            listAct.forEach(ta -> {
                if(ta.getPredecessors() != null) {
                    ta.setPredecessors(null);
                }
            });
        };
    }
}